@@ -6,7 +6,7 @@ seed <- 1
 source(paste0("", "testUtils.R"))
 
 test_that(getTestName("Simple interrutpions - No events"), {
-  if (skipLongTest) return(TRUE)
+  if (skipLongTests()) return(TRUE)
   model <- model_suite$testing$nonmem$advan4_trans4
   regFilename <- "simple_bolus" # Existing non-regression file (see testSimulateBolus.R)
   
@@ -39,7 +39,7 @@ test_that(getTestName("Simple interrutpions - No events"), {
 })
 
 test_that(getTestName("Interruptions at doses times - BW covariate - No events"), {
-  if (skipLongTest) return(TRUE)
+  if (skipLongTests()) return(TRUE)
   model <- model_suite$testing$nonmem$advan4_trans4
   equation <- model %>% find(Equation("CL"))
   model <- model %>% replace(Equation("CL", paste0(equation@rhs, "*pow(BW/70, 0.75)")))
@@ -77,7 +77,7 @@ test_that(getTestName("Interruptions at doses times - BW covariate - No events")
 })
 
 test_that(getTestName("Interruptions at doses times - BW covariate - IOV on KA - No events"), {
-  if (skipLongTest) return(TRUE)
+  if (skipLongTests()) return(TRUE)
   model <- model_suite$testing$nonmem$advan4_trans4
   equation <- model %>% find(Equation("CL"))
   model <- model %>% replace(Equation("CL", paste0(equation@rhs, "*pow(BW/70, 0.75)")))
@@ -123,7 +123,7 @@ test_that(getTestName("Interruptions at doses times - BW covariate - IOV on KA -
 
 
 test_that(getTestName("Simulate initial conditions + events"), {
-  if (skipLongTest) return(TRUE)
+  if (skipLongTests()) return(TRUE)
   model <- model_suite$testing$nonmem$advan3_trans4
   model <- model %>% add(InitialCondition(compartment=1, rhs="1000"))
   
@@ -146,7 +146,7 @@ test_that(getTestName("Simulate initial conditions + events"), {
 })
 
 test_that(getTestName("Simulate multiple arms + events"), {
-  if (skipLongTest) return(TRUE)
+  if (skipLongTests()) return(TRUE)
   model <- model_suite$testing$nonmem$advan4_trans4
   
   regFilename <- "event_in_multiple_arms"
